--------------------------------------------------------------------------------
-- Project P8  - CSD 
-- Dedicated processors 
-- https://digsys.upc.edu/csd/P08/P8.html
---------------------------------------------------
-- Example of adder + accumulator circuit 
-- Control unit circuit
-- Strategy: synchronous FSM using plan C1 (the three modules in a single file)
-- Tutorial project location:
-- https://digsys.upc.edu/csd/P08/Add_Accum/Add_Accum.html
--------------------------------------------------------------------------------
LIBRARY ieee;
USE  IEEE.STD_LOGIC_1164.all;
			
ENTITY Control_unit	IS
 		Port (	
  	    	CLK 	:	IN	STD_LOGIC;
			CD   	:	IN	STD_LOGIC;	
			ST   	:	IN	STD_LOGIC;  -- external control buttons	
			ENDF	:	IN	STD_LOGIC;  -- data-path status signal 
			
			DONE	: 	OUT  STD_LOGIC; 
			CLEAR	: 	OUT  STD_LOGIC;	 
			LOAD	: 	OUT  STD_LOGIC				
		);

END Control_unit;

ARCHITECTURE plan_C1 OF Control_unit IS
	TYPE State_type IS (Idle, ST_detected_1, Operate, ST_detected_2, 
						End_of_operation, SP_active);  
	SIGNAL current_state, next_state	: State_type ; 	

BEGIN

	-------- The state register that contains the memory cells
state_register: PROCESS (CD, CLK, next_state)
	BEGIN
						
	IF CD = '1' THEN 						
		current_state <= Idle; -- reset state where to go if a CD signal is asserted 
	ELSIF (CLK'EVENT and CLK = '1') THEN	-- Synchronous D-type flip-flops
		current_state <= next_state; 
	END IF;

END PROCESS state_register;

	-------- CC1: Combinational system for calculating next state
	-------- The truth table that contains all the state transitions (arrows)

CC_1: PROCESS (current_state, ST, ENDF)
	BEGIN
	   
		CASE current_state IS
				
			WHEN Idle =>
			   IF (ST = '1') THEN
					next_state <= ST_detected_1;	
				ELSE    
					next_state <= Idle; 
				END IF;
				
			WHEN ST_detected_1 =>  --Loop until ST is released by the user
				IF (ST = '0') THEN
					next_state <= Operate; 
				ELSE    
					next_state <= ST_detected_1 ;
				END IF;
				
			WHEN Operate => 
				IF (ST = '1') THEN
					next_state <= ST_detected_2; 
				ELSIF (ENDF = '1') THEN
					next_state <= End_of_operation ;
				ELSE
					next_state <= Operate ;	 -- continue operations		
				END IF;
				
			WHEN ST_detected_2 =>  --Loop until ST is released by the user
				IF (ST = '0') THEN
					next_state <= Idle; 
				ELSE    
					next_state <= ST_detected_2;
				END IF;
				
			WHEN End_of_operation => 		-- generate the DONE pulse
					next_state <= SP_active;
					
			WHEN SP_active => --Loop until SP is released by the user
				IF (ENDF = '0' ) THEN
					next_state <= Idle; 
				ELSE    
					next_state <= SP_active;
				END IF;
								
		END CASE ;		

END PROCESS CC_1;

	-------- CS_2: combinational system for calculating the outputs 
CC_2: PROCESS (current_state)
	BEGIN
		CASE current_state IS
			WHEN Idle => 			-- Do nothing
				LOAD 	<= '0';
				CLEAR	<= '0';		
				DONE	<= '0';		
				
			WHEN ST_detected_1 => -- Start operations
				LOAD 	<= '1';
				CLEAR	<= '0';		
				DONE	<= '0';	

			WHEN Operate => 
				LOAD 	<= '1';-- continue operations
				CLEAR	<= '0';		
				DONE	<= '0';		

			WHEN ST_detected_2 => 
				LOAD 	<= '1';
				CLEAR	<= '1';	-- Clear data and stop
				DONE	<= '0';	

			WHEN End_of_operation => 
				LOAD 	<= '0';
				CLEAR	<= '0';		
				DONE	<= '1';	-- generate a pulse for one T_CLK
				
			WHEN SP_active => 
				LOAD 	<= '1';
				CLEAR	<= '1';	 -- Clear data and stop	
				DONE	<= '0';					

		END CASE ;		

END PROCESS CC_2;

-- Place here other logic if necessary

END plan_C1;