--------------------------------------------------------------------------------
-- Project P8  - CSD 
-- Dedicated processors 
-- https://digsys.upc.edu/csd/P08/P8.html
---------------------------------------------------
-- Example of adder + accumulator circuit 
-- Data-path circuit 
-- Strategy: plan C2 using components and signals
-- Tutorial project location:
-- https://digsys.upc.edu/csd/P08/Add_Accum/Add_Accum.html
--------------------------------------------------------------------------------

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;

ENTITY Datapath IS  
       PORT(
			CD			: 	IN std_logic;
			CLK			: 	IN std_logic;
			CLR, LD		: 	IN std_logic;
			SP			: 	IN std_logic; -- Stop button
			Din			: 	IN STD_LOGIC_VECTOR(3 DOWNTO 0);
			SEGT_L		: 	OUT STD_LOGIC_VECTOR(6 DOWNTO 0);	
			SEGU_L		: 	OUT STD_LOGIC_VECTOR(6 DOWNTO 0);				
			ENDF		:	OUT std_logic	-- status flag indicator 
		);
END Datapath ;

ARCHITECTURE schematic_planC2 OF Datapath  IS  

-- NOTE: Some components are simply taken from our library of products 
-- at: https://digsys.upc.edu/csd/products/products.html
-- and they have a range larger that required in this application. 

 	COMPONENT Adder_8bit IS  -- It will be used as Adder_7bit
		PORT (
			A,B		: 	IN 		STD_LOGIC_VECTOR(7 DOWNTO 0); 				
			Cin		: 	IN 		STD_LOGIC;    
			S		: 	OUT 	STD_LOGIC_VECTOR (7 DOWNTO 0); -- result
			Cout	: 	OUT  	STD_LOGIC;
			Z		: 	OUT 	STD_LOGIC	-- Z = 1 when S(7..0) = 0 and  Cout = 0;
		  );
	END COMPONENT;
	
 	COMPONENT Quad_MUX_2 IS  
		PORT (
			CH0	:	IN STD_LOGIC_VECTOR(3 downto 0); 
			CH1	:	IN STD_LOGIC_VECTOR(3 downto 0);
			E	:	IN STD_LOGIC;
			Y	:	OUT STD_LOGIC_VECTOR(3 downto 0);
			S	:	IN STD_LOGIC 
		    );
	END COMPONENT;
	
 	COMPONENT Septuple_MUX_2 IS  
		PORT (
			CH0	:	IN STD_LOGIC_VECTOR(6 downto 0); 
			CH1	:	IN STD_LOGIC_VECTOR(6 downto 0);  
			E	:	IN STD_LOGIC;
			Y	:	OUT STD_LOGIC_VECTOR(6 downto 0); 
			S	:	IN STD_LOGIC 
		    );
	END COMPONENT;
 
	COMPONENT BCD_adapter IS  	
		PORT (
			D	:	IN	STD_LOGIC_VECTOR(3 downto 0); 
			Y	:	OUT	STD_LOGIC_VECTOR(3 downto 0)
		);	
	END COMPONENT;		

 	COMPONENT Data_Reg_4bit IS
		Port (	
			CLK, CD, LD	: IN	STD_LOGIC;
			Q			: OUT	STD_LOGIC_VECTOR(3 DOWNTO 0);
			Din			: IN 	STD_LOGIC_VECTOR(3 DOWNTO 0)
		);
	END COMPONENT;	
	
 	COMPONENT Data_Reg_7bit IS
		Port (	
			CLK, CD, LD	: IN	STD_LOGIC;
			Q			: OUT	STD_LOGIC_VECTOR(6 DOWNTO 0);
			Din			: IN 	STD_LOGIC_VECTOR(6 DOWNTO 0)
		);
	END COMPONENT;	
	
 	COMPONENT Comp_8bit IS	-- It will be used as Comp_7bit
		PORT (
			A, B		: IN	STD_LOGIC_VECTOR(7 DOWNTO 0); 
			Gi, Ei, Li	: IN	STD_LOGIC;	
			GT, EQ, LT	: OUT	STD_LOGIC    
		); 
	END COMPONENT;		
	
	COMPONENT Bin_BCD_9bit IS -- It will be used as Bin_BCD_7bit
		port (
			B	: 	in	STD_LOGIC_VECTOR(8 DOWNTO 0); 
			T, U: 	out	STD_LOGIC_VECTOR(3 DOWNTO 0);
			H	: 	out	STD_LOGIC_VECTOR(2 DOWNTO 0)
		);
	END COMPONENT;		
	
	COMPONENT Hex_7seg_decoder IS
		port (	
			X_IN			        			: IN STD_LOGIC_VECTOR(3 DOWNTO 0); 
			BI_L                   	    		: IN STD_LOGIC;      
			A_L, B_L, C_L, D_L, E_L, F_L, G_L  	: OUT STD_LOGIC
	);
	END COMPONENT;	
	
--------------------------------------------	
-- Signals for connecting components together (just the internal wires)
-- or for leaving unconnected unused outputs from components
	SIGNAL F, G, REG1, DT, DU		: 	STD_LOGIC_VECTOR(3 DOWNTO 0);
	SIGNAL H						: 	STD_LOGIC_VECTOR(2 DOWNTO 0);	
	SIGNAL K						: 	STD_LOGIC_VECTOR(7 DOWNTO 0);	
	SIGNAL J, REG2					: 	STD_LOGIC_VECTOR(6 DOWNTO 0);
	SIGNAL Cout, Z, GT, EQ, LT		: 	std_logic;
	SIGNAL B_L						: 	std_logic;	-- Blank illegal codes
--------------------------------------------------------------------------------
BEGIN  
-- Instantiation of the components 
	Chip1	: BCD_adapter 
		PORT MAP (
	-- from component name 	=> to signal or port name		
			D				=> Din,
			Y				=> F
		);
		
	Chip2	: Quad_MUX_2    
		PORT MAP (
	-- from component name 	=> to signal or port name		
			CH0				=> F,
			CH1				=> "0000", -- this is the synchronous reset
			E				=> '1',	
			Y				=> G,	
			S				=> CLR
		);

	Chip3	: Data_Reg_4bit    
		PORT MAP (
	-- from component name 	=> to signal or port name		
			CLK				=> CLK,
			CD				=> CD, -- this is the synchronous reset
			LD				=> LD,	
			Din				=> G,	
			Q				=> REG1
		);
	Chip4	: Adder_8bit
		PORT MAP (
	-- from component name 	=> to signal or port name		
			A(7 downto 4)	=>	"0000",
			A(3 downto 0)	=>	REG1,
			B(7)			=>	'0',	
			B(6 downto 0)	=>	REG2,				
			Cin				=>	'0',
			S				=>	K,	
			Cout			=>	Cout,
			Z				=>	Z			
		);
		
	Chip5	: Septuple_MUX_2    
		PORT MAP (
	-- from component name 	=> to signal or port name		
			CH0				=> K(6 downto 0),
			CH1				=> "0000000", -- this is the synchronous reset
			E				=> '1',	
			Y				=> J,	
			S				=> CLR
		);

	Chip6	: Data_Reg_7bit    
		PORT MAP (
	-- from component name 	=> to signal or port name		
			CLK				=> CLK,
			CD				=> CD, -- this is the synchronous reset
			LD				=> LD,	
			Din				=> J,	
			Q				=> REG2
		);		

	Chip7	: Comp_8bit    -- It will be used as Comp_7bit
		PORT MAP (
	-- from component name 	=> to signal or port name		
			A(6 downto 0)	=> REG2,
			A(7)			=> '0', 
			B				=> "01100011", -- number 99 			
			Gi				=> '0',	
			Ei				=> '1',	
			Li				=> '0',
			GT				=> GT,	
			EQ				=> EQ,	
			LT				=> LT			
		);	
		
	Chip8	: Bin_BCD_9bit  -- It will be used as Bin_BCD_7bit
		PORT MAP (
	-- from component name 	=> to signal or port name		
			B(6 downto 0)	=> REG2,-- only 7 bits are used 
			B(8 downto 7)	=> "00",
			H				=> H,
			T				=> DT,	
			U				=> DU	
		);		
	
	Chip9	: Hex_7seg_decoder 	-- Used as a BCD to 7-seg decoder 
		PORT MAP (
	-- from component name 	=> to signal or port name		
			X_in			=> DT,	-- decoding tens digit
			BI_L 			=> B_L, -- blanking input
			A_L				=> SEGT_L(6),
			B_L				=> SEGT_L(5),
			C_L				=> SEGT_L(4),
			D_L				=> SEGT_L(3),
			E_L				=> SEGT_L(2),
			F_L				=> SEGT_L(1),
			G_L				=> SEGT_L(0)		
		);		

	Chip10	: Hex_7seg_decoder 	-- Used as a BCD to 7-seg decoder 
		PORT MAP (
	-- from component name 	=> to signal or port name		
			X_in			=> DU,	-- decoding units digit
			BI_L 			=> B_L, -- blanking input
			A_L				=> SEGU_L(6),
			B_L				=> SEGU_L(5),
			C_L				=> SEGU_L(4),
			D_L				=> SEGU_L(3),
			E_L				=> SEGU_L(2),
			F_L				=> SEGU_L(1),
			G_L				=> SEGU_L(0)			
		);		


-- The additional logic:
B_L <= not(GT); -- to blank illegal codes when > 99 

ENDF <= GT or SP;

	
END schematic_planC2 ;
